import json
import logging
import urllib.request

from .base import Wechat

logger = logging.getLogger(__name__)


class Message(Wechat):
    """
    模板消息
    """
    def send(self, openid, template, data, url=None):
        token = self._get_access_token()
        url = 'https://api.weixin.qq.com/cgi-bin/message/template/send?access_token={}'.format(token)

        data = {
                "touser":openid,
                "template_id":template,
                "data":data
        }
        data = json.dumps(data).encode('utf8')
        with urllib.request.urlopen(url, data) as f:
            logger.info(f.read().decode('utf8'))



import logging
import random

from django.test import TestCase
from wechat.wxa import Wxa

logger = logging.getLogger(__name__)


class QrcodeTest(TestCase):
    def test(self):
        path = '/pages/index/index?promotion=1'
        wxa = Wxa()
        img = wxa.get_qrcode(path)
        self.assertTrue(img)

class DeliveryTest(TestCase):
    " 测试物流信息 "
    def test(self):
        wxa = Wxa()
        order_id = random.randint(1000, 9999)
        delivery_id = 'ZTO'
        waybill_id = ''
        res = wxa.get_delivery(delivery_id, waybill_id, order_id)
        logger.debug(res)
        self.assertEqual(res['delivery_id'], delivery_id)



import hashlib
import json
import logging
import os
import time
import urllib.request

from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage

from .base import Wechat

logger = logging.getLogger(__name__)


class Wxa(Wechat):
    """
    微信小程序
    """

    def get_pay_data(self, prepay_id):
        " 获取支付数据 "
        data = {
            'appId': self.appid,
            'timeStamp': int(time.time()),
            'nonceStr': self._make_nonce(),
            'package': 'prepay_id='+prepay_id,
            'signType': 'MD5'
        }
        data['paySign'] = self._make_sign(data)

        return data

    def get_qrcode(self, path):
        " 获取小程序二维码 "
        filename = hashlib.md5(path.encode('utf8')).hexdigest().upper()
        qrcode = 'wechat/{}.jpg'.format(filename)

        if default_storage.exists(qrcode):
            return default_storage.url(qrcode)

        token = self._get_access_token()
        url = 'https://api.weixin.qq.com/wxa/getwxacode?access_token={}'.format(token)
        data = {'path': path}
        data = json.dumps(data).encode('utf8')
        with urllib.request.urlopen(url, data) as f:
            res = f.read()
            default_storage.save(qrcode, ContentFile(res))

        return default_storage.url(qrcode)


    def get_delivery(self, delivery_id, waybill_id, order_id, openid=None):
        " 获取物流信息 "
        token = self._get_access_token()
        url = 'https://api.weixin.qq.com/cgi-bin/express/business/path/get?access_token={}'.format(token)
        data = {
            'order_id': order_id,
            'openid': openid,
            'delivery_id': delivery_id,
            'waybill_id': waybill_id
        }
        data = json.dumps(data).encode('utf8')
        with urllib.request.urlopen(url, data) as f:
            res = f.read().decode('utf8')
            res = json.loads(res)

        return res



import logging
import urllib.request

import xmltodict

from .base import Wechat

logger = logging.getLogger(__name__)

class Payment(Wechat):
    """
    支付模块
    """
    def get_prepay_id(self, data):
        "获取prepayID"
        data = {
            'xml': {
            'appid': self.appid,
            'mch_id': self.mch_id,
            'nonce_str': self._make_nonce(),
            'body': data['body'],
            'out_trade_no': data['out_trade_no'],
            'total_fee': data['total_fee'],
            'spbill_create_ip': data['spbill_create_ip'],
            'notify_url': data['notify_url'],
            'openid': data['openid'],
            'trade_type': data['trade_type']
            }
        }
        data['xml']['sign'] = self._make_sign(data['xml'])

        xmldata = xmltodict.unparse(data).encode('utf8')
        url = 'https://api.mch.weixin.qq.com/pay/unifiedorder'
        req = urllib.request.Request(url=url, data=xmldata)
        with urllib.request.urlopen(req) as f:
            result = f.read().decode('utf8')
            result = xmltodict.parse(result)

            if result['xml']['return_code'] == 'SUCCESS':
                return result['xml']['prepay_id']

        raise ValueError(result['xml'])



    def parse_result(self, result):
        " 解析异步通知返回值 "
        try:
            xmldata = xmltodict.parse(result)

            if xmldata['xml']['return_code'] == 'SUCCESS':
                is_success = True
            else:
                is_success = False

            return True, xmldata['xml']
        except Exception as e:
            logger.debug(e)

            return False, None


    def return_failure(self):
        return "<xml><return_code><![CDATA[ERROR]]></return_code><return_msg><![CDATA[ERROR]]></return_msg></xml>"

    def return_success(self):
        return "<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>"



import hashlib
import json
import logging
import random
import string
import urllib.parse
import urllib.request

from django.conf import settings
from django.core.cache import cache

logger = logging.getLogger(__name__)

class Wechat:
    """
    微信平台(开放平台, 小程序等)
    """
    def __init__(self):
        self.appid = settings.WECHAT.get('appid', None)
        self.secret = settings.WECHAT.get('secret', None)
        self.mch_id = settings.WECHAT.get('mch_id', None)
        self.key = settings.WECHAT.get('key', None)

    def _get_access_token(self):
        " 获取accesstoken "
        access_token = cache.get('wx_access_token')

        if access_token:
            return access_token
        url = 'https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={}&secret={}'.format(self.appid, self.secret)
        with urllib.request.urlopen(url) as f:
            res = f.read().decode('utf8')
            res = json.loads(res)
            cache.set(
                'wx_access_token',
                res['access_token'],
                int(res['expires_in'])
            )

            return res['access_token']


    def _make_sign(self, data):
        "生成微信签名"
        data = urllib.parse.urlencode(sorted(data.items()))
        data = urllib.parse.unquote(data)
        data += '&key=' + self.key
        data = str.encode(data)

        return hashlib.md5(data).hexdigest().upper()

    def _make_nonce(self, length=32):
        "生成随即字符串"

        return ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(length))
